package test.drools.classloader;

import java.io.InputStream;
import java.io.InputStreamReader;

import org.drools.RuleBase;
import org.drools.RuleBaseFactory;
import org.drools.StatelessSession;
import org.drools.compiler.PackageBuilder;
import org.drools.rule.Package;

public class Driver {

    private static final String RULES_DRL = "Rules.drl";

    public static void main(String[] args) throws Exception {
        
        Driver me = new Driver();
        me.go();
        
    }

    private void go() throws Exception {
        // Save original classloader
        ClassLoader original = Thread.currentThread().getContextClassLoader();
        
        // Create Classloader 1 with parent of original
        FooClassLoader loader1 = new FooClassLoader(original);
        Thread.currentThread().setContextClassLoader(loader1);
        testRules();
        
        // Create Classloader 2 with parent of original
        FooClassLoader loader2 = new FooClassLoader(original);
        Thread.currentThread().setContextClassLoader(loader2);
        testRules();
        
        // Restore
        Thread.currentThread().setContextClassLoader(original);
    }

    private void testRules() throws Exception {
        PackageBuilder builder = new PackageBuilder();
        Package pkg = buildPackage(new String[] { RULES_DRL }, builder);
        RuleBase base = RuleBaseFactory.newRuleBase();
        base.addPackage(pkg);
        StatelessSession session = base.newStatelessSession();
        final ClassLoader loader = Thread.currentThread().getContextClassLoader();
        Object inst = loader.loadClass(FooClassLoader.FOO_CLASSNAME).newInstance();
        session.execute(new Object[] { inst });
    }
    
    private Package buildPackage(String[] rulesResources, PackageBuilder builder) {
        try {
            for (String rulesResource : rulesResources) {
                InputStream in = getClass().getResourceAsStream(rulesResource);
                builder.addPackageFromDrl(new InputStreamReader(in));
            }
            return builder.getPackage();
        }
        catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
    
}
